#include "jaycam.h"

void resetnic(){
  int i, count;

  outnic(CMDR, 0x21);  // Abort and DMA and stop the NIC
  delay(10);

  outnic(DCR, DCRVAL);   // Set word-wide access

  outnic(RBCR0, 0x00); // Clear the count regs
  outnic(RBCR1, 0x00);

  outnic(IMR, 0x00);   // Mask completion irq
  outnic(ISR, 0xFF);   // clear interrupt status register

  outnic(RCR, 0x20);   // 0x20  Set to monitor mode
  outnic(TCR, 0x02);  // put nic in normal operation

  // Set Rx start, Rx stop, Boundary and TX start regs
  outnic(PSTART, RXSTART);
  outnic(PSTOP, RXSTOP);
  outnic(BNRY, (BYTE)(RXSTOP-1));
  outnic(TPSR, TXSTART);
  
  outnic(ISR, 0xFF);   // clear interrupt status register
  outnic(IMR, 0x00);   // Mask completion irq

  // put nic in start mode
  outnic(CMDR, 0x22); // start nic
  outnic(TCR, 0x00);  // put nic in normal operation
}

int init_etherne()
{
  outnic(NE_RESET, innic(NE_RESET));  // Do reset
  delay(2);
  if ((innic(ISR) & 0x80) == 0)       // Report if failed
  {
    print("  Ethernet card failed to reset!\r\n");
    return 0;
  }
  else
  {
    print("Ethernet card reset successful...");
    resetnic(); // Reset Ethernet card,
    print("Ethernet card intialization complete!\r\n");
  }
  return 1;
}

// diagnostic that tests nic functionality
void diag(){
  int i;
  
  print("\r\n**************** Beginning of Diagnostic ****************\r\n\r\n");

  print("         Command Register Page Switching Test\r\n");
  print("Switching to page 1\r\n");
  outnic(CMDR, 0x61);

  print("Writing to and reading from 0x0D (value should be 0x4e): ");
  outnic(0x0D*2,0x4d);
  putnum(innic(0x0D*2));
  print("\r\n");

  print("Switching to page 0\r\n");
  outnic(CMDR, 0x21);

  print("Reading from reg offset 0x0D: ");
  putnum(innic(0x0D*2));
  print("\r\n");

  print("Switching to page 1\r\n");
  outnic(CMDR, 0x61);
  print("Reading from reg offset 0x0D (should be 0x4e): ");
  putnum(innic(0x0D*2));
  print("\r\n\r\n");

  print("         Default Value Test\r\n");
  print("Switching to page 0\r\n");
  outnic(CMDR,21);
  print("Reading from 0x16 (value should be 0x15): ");
  putnum(innic(0x16*2));
  print("\r\n");
  print("Reading from 0x12 (value should be 0x0c): ");
  putnum(innic(0x12*2));
  print("\r\n");
  print("Reading from 0x13 (value should be 0x12): ");
  putnum(innic(0x13*2));
  print("\r\n\r\n**************** End of Diagnostic ****************\r\n\r\n");
}

void delay(int mult){
  int i;
  int delay = 1000000*mult;
  for(i=0; i<delay; i++);
}

void initpacket(Xuint8 *packet)
{

  packet[0]=0xFF;
  packet[1]=0xFF;
  packet[2]=0xFF;
  packet[3]=0xFF;
  packet[4]=0xFF;
  packet[5]=0xFF;
  packet[6]=0x00;
  packet[6]=0x0D;
  packet[8]=0x60;
  packet[9]=0x7F;
  packet[10]=0xF9;
  packet[11]=0xAF;
  packet[12]=0x08;
  packet[13]=0x00;
  packet[14]=0x45;
  packet[15]=0x00;
  packet[16]=0x00;
  packet[17]=0xBE;
  packet[18]=0x00;
  packet[19]=0x00;
  packet[20]=0x00;
  packet[21]=0x00;
  packet[22]=0x01;
  packet[23]=0x11;
  packet[24]=0xB8;
  packet[25]=0xEE;
  packet[26]=0x80;
  packet[27]=0x3B;
  packet[28]=0x95;
  packet[29]=0xF9;
  packet[30]=0xE4;
  packet[31]=0x05;
  packet[32]=0x06;
  packet[33]=0x07;
  packet[34]=0x0C;
  packet[35]=0x2C;
  packet[36]=0x1A;
  packet[37]=0x85;
  packet[38]=0x00;
  packet[39]=0xAA;
  packet[40]=0x00;
  packet[41]=0x00;
  //extra byte
  packet[42]=0x00;
  

/*   // create a broadcast packet */
/*   packet[0] = 0xff; */
/*   packet[1] = 0xff; */
/*   packet[2] = 0xff; */
/*   packet[3] = 0xff; */
/*   packet[4] = 0xff; */
/*   packet[5] = 0xff; */

/*   // source addr */
/*   packet[6] = 0x01; */
/*   packet[7] = 0x02; */
/*   packet[8] = 0x03; */
/*   packet[9] = 0x04; */
/*   packet[10] = 0x05; */
/*   packet[11] = 0x06; */

/*   // length field */
/*   packet[12] = 0x6a; */
/*   packet[13] = 0x05; */

/*   // extra space */
/*   packet[14] = 0; */

}

