#include "xio.h"
#include "xbasic_types.h"

#define BYTE unsigned char
#define WORD unsigned short

#define NIC_ADDR 0x00A00400     /* IO Base Address */
#define WORDMODE 1              /* Set to zero if using 8-bit XT-bus cards */

/* NE2000 definitions */
#define DATAPORT (0x10*2)
#define NE_RESET (0x1f*2)

/* 8390 Network Interface Controller (NIC) page0 register offsets */
#define CMDR    (0x00*2)            /* command register for read & write */
#define PSTART  (0x01*2)            /* page start register for write */
#define PSTOP   (0x02*2)            /* page stop register for write */
#define BNRY    (0x03*2)            /* boundary reg for rd and wr */
#define TPSR    (0x04*2)            /* tx start page start reg for wr */
#define TBCR0   (0x05*2)            /* tx byte count 0 reg for wr */
#define TBCR1   (0x06*2)            /* tx byte count 1 reg for wr */
#define ISR     (0x07*2)            /* interrupt status reg for rd and wr */
#define RSAR0   (0x08*2)            /* low byte of remote start addr */
#define RSAR1   (0x09*2)            /* hi byte of remote start addr */
#define RBCR0   (0x0A*2)            /* remote byte count reg 0 for wr */
#define RBCR1   (0x0B*2)            /* remote byte count reg 1 for wr */
#define RCR     (0x0C*2)            /* rx configuration reg for wr */
#define TCR     (0x0D*2)            /* tx configuration reg for wr */
#define DCR     (0x0E*2)            /* data configuration reg for wr */
#define IMR     (0x0F*2)            /* interrupt mask reg for wr */
#define IFG     (0x16*2)            /* Inter-frame gap register */
#define IFGS1   (0x12*2)            /* Inter-frame gap Segment 1 register */
#define IFGS2   (0x13*2)            /* Inter-frame gap Segment 2 register */

/* NIC page 1 register offsets */
#define PAR0    (0x01*2)            /* physical addr reg 0 for rd and wr */
#define CURR    (0x07*2)            /* current page reg for rd and wr */
#define MAR0    (0x08*2)            /* multicast addr reg 0 for rd and WR */

/* Buffer Length and Field Definition Info */
#define TXSTART  0x40           /* Tx buffer start page */
#define TXPAGES  6              /* Pages for Tx buffer */
#define RXSTART  (TXSTART+TXPAGES)  /* Rx buffer start page */
#if WORDMODE
#define RXSTOP   0x7e           /* Rx buffer end page for word mode */
#define DCRVAL   0x49           /* DCR values for word mode */
#else
#define RXSTOP   0x5f           /* Ditto for byte mode */
#define DCRVAL   0x48
#endif
#define STARHACK 0              /* Set non-zero to enable Starlan length hack */

void init();
void getnic(WORD addr, BYTE data[], WORD len);
void putnic(WORD addr, BYTE data[], WORD len);
void delay();

int main()
{
  int i=0;
  WORD len;
  WORD pkt[8],pkt2[8];

  print("\r\n");
  XIo_Out16(NIC_ADDR+NE_RESET, XIo_In16(NIC_ADDR+NE_RESET));
  // report failed
  if ((XIo_In16(NIC_ADDR+ISR) & 0x80) == 0){
      print("Ethernet card failed to reset!\r\n");
  }
  else{
    print("Ethernet card reset succesful!\r\n");
  }

  XIo_Out16(NIC_ADDR+PSTART, 0x0040);
  XIo_Out16(NIC_ADDR,0x0021);
  print("\r\nPSTART ");
  putnum(XIo_In16(NIC_ADDR+PSTART));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+IFG));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+IFGS1));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+IFGS2));

  print("\r\nCMD ");
  putnum(XIo_In16(NIC_ADDR));
  print("\r\nISR ");
  putnum(XIo_In16(NIC_ADDR+ISR));
  print("\r\n");
  print("Enable interrupts, init DCR, start dma, and give start command");
  XIo_Out16(NIC_ADDR+IMR,0x000ff); //enable interrupts
  XIo_Out16(NIC_ADDR+DCR,0x0081); //set DCR
  XIo_Out16(NIC_ADDR,0x0010); //remote DMA op
  print("\r\nISR after dma start ");
  putnum(XIo_In16(NIC_ADDR+ISR));
  XIo_Out16(NIC_ADDR,0x0022); //start command
  print("\r\nISR after start ");
  putnum(XIo_In16(NIC_ADDR+ISR));
  XIo_Out16(NIC_ADDR,0x0021); //stop command
  print("\r\nISR after stop ");
  putnum(XIo_In16(NIC_ADDR+ISR));
  print("\r\n");

  print("\r\nCMD ");
  putnum(XIo_In16(NIC_ADDR));
  print("\r\n");

  XIo_Out16(NIC_ADDR+NE_RESET, XIo_In16(NIC_ADDR+NE_RESET));

  print("\r\nCMD ");
  XIo_Out16(NIC_ADDR+CMDR,0x0061); // switch to page 1
  putnum(XIo_In16(NIC_ADDR));
  print("\r\n");

  print("Write and Read physical address from registers\r\n");
  XIo_Out16(NIC_ADDR+2,0x10);
  XIo_Out16(NIC_ADDR+4,0xff);
  XIo_Out16(NIC_ADDR+6,0xff);
  XIo_Out16(NIC_ADDR+8,0xff);
  XIo_Out16(NIC_ADDR+10,0xff);
  XIo_Out16(NIC_ADDR+12,0x54);
  
  print("\r\nCMD ");
  putnum(XIo_In16(NIC_ADDR));
  print("\r\n");  

  putnum(XIo_In16(NIC_ADDR+2));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+4));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+6));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+8));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+10));
  print("\r\n");
  putnum(XIo_In16(NIC_ADDR+12));
  print("\r\n");

  print("\r\nCMD ");
  putnum(XIo_In16(NIC_ADDR));
  print("\r\n");  

  return;

  print("\r\n");
  init();

  pkt[0] = 254;
  pkt[1] = 254;
  pkt[2] = 3;
  pkt[3] = 12;
  pkt[4] = 253;
  pkt[5] = 253;
  pkt[6] = 2;
  pkt[7] = 11;
  pkt2[0] = 6;
  pkt2[1] = 7;
  pkt2[2] = 8;
  pkt2[3] = 9;
  pkt2[4] = 10;
  pkt2[5] = 11;
  pkt2[6] = 12;
  pkt2[7] = 13;
  len =8;
  putnic(0x4000, pkt, len);
  delay();
  getnic(0, pkt2, len);
  for(i=0; i<len;i++){
    putnum(pkt2[i]);
    print("\r\n");
  }
  return 0;
} // end of main()

void init(){
  int cold = 1;
  int i;
  BYTE ethAddr[6];
  BYTE temp[12];
  

  XIo_Out16(NIC_ADDR+CMDR, 0x21);                 /* Stop, DMA abort, page 0 */
  delay();
  XIo_Out16(NIC_ADDR+DCR, DCRVAL);
  XIo_Out16(NIC_ADDR+RBCR0, 0);                   /* Clear remote byte count */
  XIo_Out16(NIC_ADDR+RBCR1, 0);
  XIo_Out16(NIC_ADDR+RCR, 0x20);                  /* Rx monitor mode */
  XIo_Out16(NIC_ADDR+TCR, 0x02);                  /* Tx internal loopback */
  XIo_Out16(NIC_ADDR+TPSR, TXSTART);              /* Set Tx start page */
  XIo_Out16(NIC_ADDR+PSTART, RXSTART);            /* Set Rx start, stop, boundary */
  XIo_Out16(NIC_ADDR+PSTOP, RXSTOP);
  XIo_Out16(NIC_ADDR+BNRY, (BYTE)(RXSTOP-1));
  XIo_Out16(NIC_ADDR+ISR, 0xff);                  /* Clear interrupt flags */
  XIo_Out16(NIC_ADDR+IMR, 0);                     /* Mask all interrupts */
  if (cold)
    {
      XIo_Out16(NIC_ADDR+CMDR, 0x22);             /* Start NIC, DMA abort */
      getnic(0, temp, 12);                        /* Get 6-byte addr */
      print("\r\nMAC Address\r\n");
      for (i=0; i<6; i++){                         /* Convert addr words to bytes */
      	ethAddr[i] = temp[WORDMODE ? i+i : i];
	putnum(ethAddr[i]);
	print("\r\n");
      }      
    }
  XIo_Out16(NIC_ADDR+CMDR, 0x61);                 /* Stop, DMA abort, page 1 */
  delay();
  for (i=0; i<6; i++)                             /* Set Phys addr */
    XIo_Out16(NIC_ADDR+PAR0+(i*2), ethAddr[i]);
  for (i=0; i<8; i++)                             /* Multicast accept-all */
    XIo_Out16(NIC_ADDR+MAR0+(i*2), 0xff);
  XIo_Out16(NIC_ADDR+CURR, RXSTART+1);            /* Set current Rx page */
  XIo_Out16(NIC_ADDR+CMDR, 0x20);                 /* DMA abort, page 0 */
  XIo_Out16(NIC_ADDR+RCR, 0x04);                  /* Allow broadcasts, maybe all pkts */
  XIo_Out16(NIC_ADDR+TCR, 0);                     /* Normal Tx operation */
  XIo_Out16(NIC_ADDR+ISR, 0xff);                  /* Clear interrupt flags */
  XIo_Out16(NIC_ADDR+CMDR, 0x22);
} // end of init()

// Get a packet from a given address in the NIC's RAM
void getnic(WORD addr, BYTE data[], WORD len)
{
  int i;
  register int count;
  register WORD temp;
  register WORD *dataw, dataport;
  
  count = WORDMODE ? len>>1 : len;           /* Halve byte count if word I/P */
  dataport = NIC_ADDR + DATAPORT;            /* Address of NIC data port */
  XIo_Out16(NIC_ADDR+ISR, 0x40);             /* Clear remote DMA interrupt flag */
  XIo_Out16(NIC_ADDR+RBCR0, len&0xff);       /* Byte count */
  XIo_Out16(NIC_ADDR+RBCR1, len>>8);
  XIo_Out16(NIC_ADDR+RSAR0, addr&0xff);      /* Data addr */
  XIo_Out16(NIC_ADDR+RSAR1, addr>>8);
  XIo_Out16(NIC_ADDR+CMDR, 0x0a);            /* Start, DMA remote read */
#if WORDMODE
  dataw = (WORD *)data;                      /* Use pointer for speed */
  while(count--){                            /* Get words */
    *dataw++ = XIo_In16(dataport);
  }

  //if (len & 1)                               /* If odd length, do last byte */
  //  *(BYTE *)dataw = XIo_In16(dataport);
#else
  while(count--)                             /* Get bytes */
    *data++ = XIo_In16(dataport);
#endif
  count = 1000000;
  while(count && (temp=XIo_In16(NIC_ADDR+ISR)&0x40) == 0)
    count--;
}

/* Put a packet into a given address in the NIC's RAM */
void putnic(WORD addr, BYTE data[], WORD len)
{
    register int count;
    register WORD *dataw, dataport;

    len += len & 1;                     /* Round length up to an even value */
    count = WORDMODE ? len>>1 : len;    /* Halve byte count if word O/P */
    dataport = NIC_ADDR + DATAPORT;        /* Address of NIC data port */
    XIo_Out16(NIC_ADDR+ISR, 0x40);                  /* Clear remote DMA interrupt flag */
    XIo_Out16(NIC_ADDR+RBCR0, len&0xff);            /* Byte count */
    XIo_Out16(NIC_ADDR+RBCR1, len>>8);
    XIo_Out16(NIC_ADDR+RSAR0, addr&0xff);           /* Data addr */
    XIo_Out16(NIC_ADDR+RSAR1, addr>>8);
    XIo_Out16(NIC_ADDR+CMDR, 0x12);                 /* Start, DMA remote write */
#if WORDMODE                            /* Word transfer? */
    dataw = (WORD *)data;
    while(count--)
      XIo_Out16(dataport, *dataw++);  /* O/P words */
#else
    while(count--)                  /* O/P bytes */
        XIo_Out16(dataport, *data++);
#endif
    count = 10000;                      /* Done: must ensure DMA complete */
    while(count && (XIo_In16(NIC_ADDR+ISR)&0x40)==0)
      count--;
}


void delay(){
  int i;
  for(i=0; i<50000000; i++);
} // end of delay()
